@@ -25,6 +25,7 @@ import nova.monkey_patch  # noqa
 
 import abc
 import collections
+import contextlib
 import copy
 import datetime
 import inspect
@@ -53,7 +54,6 @@ from six.moves import builtins
 from sqlalchemy.dialects import sqlite
 import testtools
 
-from nova.api.openstack import wsgi_app
 from nova.compute import rpcapi as compute_rpcapi
 from nova import context
 from nova.db.sqlalchemy import api as sqlalchemy_api
@@ -68,11 +68,6 @@ from nova.tests.unit import policy_fixture
 from nova import utils
 from nova.virt import images
 
-if six.PY2:
-    import contextlib2 as contextlib
-else:
-    import contextlib
-
 CONF = cfg.CONF
 
 logging.register_options(CONF)
@@ -144,7 +139,7 @@ class NovaExceptionReraiseFormatError(object):
     def _wrap_log_exception(self):
         exc_info = sys.exc_info()
         NovaExceptionReraiseFormatError.real_log_exception(self)
-        six.reraise(*exc_info)
+        raise exc_info[1]
 
 
 # NOTE(melwitt) This needs to be done at import time in order to also catch
@@ -288,9 +283,7 @@ class TestCase(base.BaseTestCase):
         quota.UID_QFD_POPULATED_CACHE_BY_PROJECT = set()
         quota.UID_QFD_POPULATED_CACHE_ALL = False
 
-        # make sure that the wsgi app is fully initialized for all testcase
-        # instead of only once initialized for test worker
-        wsgi_app.init_global_data.reset()
+        self.useFixture(nova_fixtures.GenericPoisonFixture())
 
     def _setup_cells(self):
         """Setup a normal cellsv2 environment.
@@ -646,8 +639,9 @@ class TestCase(base.BaseTestCase):
                          baseclass)
 
         for name in sorted(implmethods.keys()):
-            baseargs = utils.getargspec(basemethods[name])
-            implargs = utils.getargspec(implmethods[name])
+            # NOTE(stephenfin): We ignore type annotations
+            baseargs = utils.getargspec(basemethods[name])[:-1]
+            implargs = utils.getargspec(implmethods[name])[:-1]
 
             self.assertEqual(baseargs, implargs,
                              "%s args don't match base class %s" %
@@ -669,8 +663,7 @@ class APICoverage(object):
             testtools.matchers.ContainsAll(api_methods))
 
 
-@six.add_metaclass(abc.ABCMeta)
-class SubclassSignatureTestCase(testtools.TestCase):
+class SubclassSignatureTestCase(testtools.TestCase, metaclass=abc.ABCMeta):
     """Ensure all overridden methods of all subclasses of the class
     under test exactly match the signature of the base class.
 
@@ -771,12 +764,6 @@ class NoDBTestCase(TestCase):
     USES_DB = False
 
 
-class BaseHookTestCase(NoDBTestCase):
-    def assert_has_hook(self, expected_name, func):
-        self.assertTrue(hasattr(func, '__hook_name__'))
-        self.assertEqual(expected_name, func.__hook_name__)
-
-
 class MatchType(object):
     """Matches any instance of a specified type
 
