@@ -31,7 +31,6 @@ from oslo_context import context as common_context
 from oslo_context import fixture as context_fixture
 from oslo_utils import encodeutils
 from oslo_utils import fixture as utils_fixture
-from oslo_utils import units
 import six
 
 from nova import context
@@ -226,6 +225,12 @@ class GenericUtilsTestCase(test.NoDBTestCase):
         utils.ssh_execute('remotehost', 'ls', '-l')
         mock_execute.assert_called_once_with(*expected_args)
 
+    @mock.patch('nova.utils.generate_uid')
+    def test_tpool_execute(self, mock_generate):
+        expected_kargs = {'size': 12}
+        utils.tpool_execute(utils.generate_uid, 'mytopic', size=12)
+        mock_generate.assert_called_once_with('mytopic', **expected_kargs)
+
     def test_generate_hostid(self):
         host = 'host'
         project_id = '9b9e3c847e904b0686e8ffb20e4c6381'
@@ -732,43 +737,6 @@ class GetImageFromSystemMetadataTestCase(test.NoDBTestCase):
             self.assertNotIn(key, image)
 
 
-class GetImageMetadataFromVolumeTestCase(test.NoDBTestCase):
-    def test_inherit_image_properties(self):
-        properties = {"fake_prop": "fake_value"}
-        volume = {"volume_image_metadata": properties}
-        image_meta = utils.get_image_metadata_from_volume(volume)
-        self.assertEqual(properties, image_meta["properties"])
-
-    def test_image_size(self):
-        volume = {"size": 10}
-        image_meta = utils.get_image_metadata_from_volume(volume)
-        self.assertEqual(10 * units.Gi, image_meta["size"])
-
-    def test_image_status(self):
-        volume = {}
-        image_meta = utils.get_image_metadata_from_volume(volume)
-        self.assertEqual("active", image_meta["status"])
-
-    def test_values_conversion(self):
-        properties = {"min_ram": "5", "min_disk": "7"}
-        volume = {"volume_image_metadata": properties}
-        image_meta = utils.get_image_metadata_from_volume(volume)
-        self.assertEqual(5, image_meta["min_ram"])
-        self.assertEqual(7, image_meta["min_disk"])
-
-    def test_suppress_not_image_properties(self):
-        properties = {"min_ram": "256", "min_disk": "128",
-                      "image_id": "fake_id", "image_name": "fake_name",
-                      "container_format": "ami", "disk_format": "ami",
-                      "size": "1234", "checksum": "fake_checksum"}
-        volume = {"volume_image_metadata": properties}
-        image_meta = utils.get_image_metadata_from_volume(volume)
-        self.assertEqual({}, image_meta["properties"])
-        self.assertEqual(0, image_meta["size"])
-        # volume's properties should not be touched
-        self.assertNotEqual({}, properties)
-
-
 class SafeTruncateTestCase(test.NoDBTestCase):
     def test_exception_to_dict_with_long_message_3_bytes(self):
         # Generate Chinese byte string whose length is 300. This Chinese UTF-8
@@ -1326,101 +1294,3 @@ class TestOldComputeCheck(test.NoDBTestCase):
             'not allowed to directly access the database. You should run this '
             'service without the [api_database]/connection config option. The '
             'service version check will only query the local cell.')
-
-
-class RunOnceTests(test.NoDBTestCase):
-
-    fake_logger = mock.MagicMock()
-
-    @utils.run_once("already ran once", fake_logger)
-    def dummy_test_func(self, fail=False):
-        if fail:
-            raise ValueError()
-        return True
-
-    def setUp(self):
-        super(RunOnceTests, self).setUp()
-        self.dummy_test_func.reset()
-        RunOnceTests.fake_logger.reset_mock()
-
-    def test_wrapped_funtions_called_once(self):
-        self.assertFalse(self.dummy_test_func.called)
-        result = self.dummy_test_func()
-        self.assertTrue(result)
-        self.assertTrue(self.dummy_test_func.called)
-
-        # assert that on second invocation no result
-        # is returned and that the logger is invoked.
-        result = self.dummy_test_func()
-        RunOnceTests.fake_logger.assert_called_once()
-        self.assertIsNone(result)
-
-    def test_wrapped_funtions_called_once_raises(self):
-        self.assertFalse(self.dummy_test_func.called)
-        self.assertRaises(ValueError, self.dummy_test_func, fail=True)
-        self.assertTrue(self.dummy_test_func.called)
-
-        # assert that on second invocation no result
-        # is returned and that the logger is invoked.
-        result = self.dummy_test_func()
-        RunOnceTests.fake_logger.assert_called_once()
-        self.assertIsNone(result)
-
-    def test_wrapped_funtions_can_be_reset(self):
-        # assert we start with a clean state
-        self.assertFalse(self.dummy_test_func.called)
-        result = self.dummy_test_func()
-        self.assertTrue(result)
-
-        self.dummy_test_func.reset()
-        # assert we restored a clean state
-        self.assertFalse(self.dummy_test_func.called)
-        result = self.dummy_test_func()
-        self.assertTrue(result)
-
-        # assert that we never called the logger
-        RunOnceTests.fake_logger.assert_not_called()
-
-    def test_reset_calls_cleanup(self):
-        mock_clean = mock.Mock()
-
-        @utils.run_once("already ran once", self.fake_logger,
-                        cleanup=mock_clean)
-        def f():
-            pass
-
-        f()
-        self.assertTrue(f.called)
-
-        f.reset()
-        self.assertFalse(f.called)
-        mock_clean.assert_called_once_with()
-
-    def test_clean_is_not_called_at_reset_if_wrapped_not_called(self):
-        mock_clean = mock.Mock()
-
-        @utils.run_once("already ran once", self.fake_logger,
-                        cleanup=mock_clean)
-        def f():
-            pass
-
-        self.assertFalse(f.called)
-
-        f.reset()
-        self.assertFalse(f.called)
-        self.assertFalse(mock_clean.called)
-
-    def test_reset_works_even_if_cleanup_raises(self):
-        mock_clean = mock.Mock(side_effect=ValueError())
-
-        @utils.run_once("already ran once", self.fake_logger,
-                        cleanup=mock_clean)
-        def f():
-            pass
-
-        f()
-        self.assertTrue(f.called)
-
-        self.assertRaises(ValueError, f.reset)
-        self.assertFalse(f.called)
-        mock_clean.assert_called_once_with()
