@@ -40,7 +40,6 @@ from nova import profiler
 from nova import utils
 from nova.virt.libvirt import config as vconfig
 from nova.virt.libvirt import designer
-from nova.virt.libvirt import utils as libvirt_utils
 from nova.virt import osinfo
 
 
@@ -48,16 +47,6 @@ LOG = logging.getLogger(__name__)
 
 CONF = nova.conf.CONF
 
-# setting interface mtu was intoduced in libvirt 3.3, We also need to
-# check for QEMU that because libvirt is configuring in same time
-# host_mtu for virtio-net, fails if not supported.
-MIN_LIBVIRT_INTERFACE_MTU = (3, 3, 0)
-MIN_QEMU_INTERFACE_MTU = (2, 9, 0)
-
-# virtio-net.tx_queue_size support
-MIN_LIBVIRT_TX_QUEUE_SIZE = (3, 7, 0)
-MIN_QEMU_TX_QUEUE_SIZE = (2, 10, 0)
-
 SUPPORTED_VIF_MODELS = {
     'qemu': [
         network_model.VIF_MODEL_VIRTIO,
@@ -67,7 +56,8 @@ SUPPORTED_VIF_MODELS = {
         network_model.VIF_MODEL_E1000,
         network_model.VIF_MODEL_E1000E,
         network_model.VIF_MODEL_LAN9118,
-        network_model.VIF_MODEL_SPAPR_VLAN],
+        network_model.VIF_MODEL_SPAPR_VLAN,
+        network_model.VIF_MODEL_VMXNET3],
     'kvm': [
         network_model.VIF_MODEL_VIRTIO,
         network_model.VIF_MODEL_NE2K_PCI,
@@ -75,7 +65,8 @@ SUPPORTED_VIF_MODELS = {
         network_model.VIF_MODEL_RTL8139,
         network_model.VIF_MODEL_E1000,
         network_model.VIF_MODEL_E1000E,
-        network_model.VIF_MODEL_SPAPR_VLAN],
+        network_model.VIF_MODEL_SPAPR_VLAN,
+        network_model.VIF_MODEL_VMXNET3],
     'xen': [
         network_model.VIF_MODEL_NETFRONT,
         network_model.VIF_MODEL_NE2K_PCI,
@@ -185,7 +176,7 @@ class LibvirtGenericVIFDriver(object):
         return model
 
     def get_base_config(self, instance, mac, image_meta,
-                        inst_type, virt_type, vnic_type, host):
+                        inst_type, virt_type, vnic_type):
         # TODO(sahid): We should rewrite it. This method handles too
         # many unrelated things. We probably need to have a specific
         # virtio, vhost, vhostuser functions.
@@ -204,6 +195,8 @@ class LibvirtGenericVIFDriver(object):
                 conf, mac, model, driver, vhost_queues, rx_queue_size)
             return conf
 
+        rx_queue_size = CONF.libvirt.rx_queue_size
+
         # if model has already been defined,
         # image_meta contents will override it
         model = self.get_vif_model(image_meta=image_meta, vif_model=model)
@@ -236,8 +229,7 @@ class LibvirtGenericVIFDriver(object):
 
         if driver == 'vhost' or driver is None:
             # vhost backend only supports update of RX queue size
-            rx_queue_size, _ = self._get_virtio_queue_sizes(host)
-            if rx_queue_size:
+            if CONF.libvirt.rx_queue_size:
                 # TODO(sahid): Specifically force driver to be vhost
                 # that because if None we don't generate the XML
                 # driver element needed to set the queue size
@@ -317,25 +309,10 @@ class LibvirtGenericVIFDriver(object):
         return (("qvb%s" % iface_id)[:network_model.NIC_NAME_LEN],
                 ("qvo%s" % iface_id)[:network_model.NIC_NAME_LEN])
 
-    def _set_mtu_config(self, vif, host, conf):
-        """:param vif: nova.network.modle.vif
-           :param host: nova.virt.libvirt.host.Host
-           :param conf: nova.virt.libvirt.config.LibvirtConfigGuestInterface
-        """
-        network = vif.get('network')
-        if (network and network.get_meta("mtu") and
-                self._has_min_version_for_mtu(host)):
-            designer.set_vif_mtu_config(conf, network.get_meta("mtu"))
-
-    def _has_min_version_for_mtu(self, host):
-        return host.has_min_version(MIN_LIBVIRT_INTERFACE_MTU,
-                                    MIN_QEMU_INTERFACE_MTU)
-
     def get_config_802qbg(self, instance, vif, image_meta,
-                          inst_type, virt_type, host):
+                          inst_type, virt_type):
         conf = self.get_base_config(instance, vif['address'], image_meta,
-                                    inst_type, virt_type, vif['vnic_type'],
-                                    host)
+                                    inst_type, virt_type, vif['vnic_type'])
 
         params = vif["qbg_params"]
         designer.set_vif_host_backend_802qbg_config(
@@ -350,10 +327,9 @@ class LibvirtGenericVIFDriver(object):
         return conf
 
     def get_config_802qbh(self, instance, vif, image_meta,
-                          inst_type, virt_type, host):
+                          inst_type, virt_type):
         conf = self.get_base_config(instance, vif['address'], image_meta,
-                                    inst_type, virt_type, vif['vnic_type'],
-                                    host)
+                                    inst_type, virt_type, vif['vnic_type'])
 
         profile = vif["profile"]
         vif_details = vif["details"]
@@ -370,10 +346,9 @@ class LibvirtGenericVIFDriver(object):
         return conf
 
     def get_config_hw_veb(self, instance, vif, image_meta,
-                            inst_type, virt_type, host):
+                          inst_type, virt_type):
         conf = self.get_base_config(instance, vif['address'], image_meta,
-                                    inst_type, virt_type, vif['vnic_type'],
-                                    host)
+                                    inst_type, virt_type, vif['vnic_type'])
 
         profile = vif["profile"]
         vif_details = vif["details"]
@@ -390,14 +365,13 @@ class LibvirtGenericVIFDriver(object):
         return conf
 
     def get_config_hostdev_physical(self, instance, vif, image_meta,
-                                    inst_type, virt_type, host):
+                                    inst_type, virt_type):
         return self.get_base_hostdev_pci_config(vif)
 
     def get_config_macvtap(self, instance, vif, image_meta,
-                           inst_type, virt_type, host):
+                           inst_type, virt_type):
         conf = self.get_base_config(instance, vif['address'], image_meta,
-                                    inst_type, virt_type, vif['vnic_type'],
-                                    host)
+                                    inst_type, virt_type, vif['vnic_type'])
 
         vif_details = vif['details']
         macvtap_src = vif_details.get(network_model.VIF_DETAILS_MACVTAP_SOURCE)
@@ -426,96 +400,71 @@ class LibvirtGenericVIFDriver(object):
         return conf
 
     def get_config_iovisor(self, instance, vif, image_meta,
-                           inst_type, virt_type, host):
+                           inst_type, virt_type):
         conf = self.get_base_config(instance, vif['address'], image_meta,
-                                    inst_type, virt_type, vif['vnic_type'],
-                                    host)
+                                    inst_type, virt_type, vif['vnic_type'])
 
         dev = self.get_vif_devname(vif)
-        designer.set_vif_host_backend_ethernet_config(conf, dev, host)
+        designer.set_vif_host_backend_ethernet_config(conf, dev)
 
         designer.set_vif_bandwidth_config(conf, inst_type)
 
         return conf
 
     def get_config_midonet(self, instance, vif, image_meta,
-                           inst_type, virt_type, host):
+                           inst_type, virt_type):
         conf = self.get_base_config(instance, vif['address'], image_meta,
-                                    inst_type, virt_type, vif['vnic_type'],
-                                    host)
+                                    inst_type, virt_type, vif['vnic_type'])
 
         dev = self.get_vif_devname(vif)
-        designer.set_vif_host_backend_ethernet_config(conf, dev, host)
+        designer.set_vif_host_backend_ethernet_config(conf, dev)
 
         return conf
 
-    def get_config_tap(self, instance, vif, image_meta,
-                       inst_type, virt_type, host):
+    def get_config_tap(self, instance, vif, image_meta, inst_type, virt_type):
         conf = self.get_base_config(instance, vif['address'], image_meta,
-                                    inst_type, virt_type, vif['vnic_type'],
-                                    host)
+                                    inst_type, virt_type, vif['vnic_type'])
 
         dev = self.get_vif_devname(vif)
-        designer.set_vif_host_backend_ethernet_config(conf, dev, host)
+        designer.set_vif_host_backend_ethernet_config(conf, dev)
 
-        self._set_mtu_config(vif, host, conf)
+        network = vif.get('network')
+        if network and network.get_meta('mtu'):
+            designer.set_vif_mtu_config(conf, network.get_meta("mtu"))
 
         return conf
 
     def get_config_ib_hostdev(self, instance, vif, image_meta,
-                              inst_type, virt_type, host):
+                              inst_type, virt_type):
         return self.get_base_hostdev_pci_config(vif)
 
-    def _get_virtio_queue_sizes(self, host):
-        """Returns rx/tx queue sizes configured or (None, None)
-
-        Based on tx/rx queue sizes configured on host (nova.conf). The
-        methods check whether the versions of libvirt and QEMU are
-        corrects.
-        """
-        # TODO(sahid): For vhostuser interface this function is called
-        # from get_base_config and also from the method reponsible to
-        # configure vhostuser interface meaning that the logs can be
-        # duplicated. In future we want to rewrite get_base_config.
-        rx, tx = CONF.libvirt.rx_queue_size, CONF.libvirt.tx_queue_size
-        if tx and not host.has_min_version(
-                MIN_LIBVIRT_TX_QUEUE_SIZE, MIN_QEMU_TX_QUEUE_SIZE):
-            LOG.warning('Setting TX queue size requires libvirt %s and QEMU '
-                        '%s version or greater.',
-                        libvirt_utils.version_to_string(
-                            MIN_LIBVIRT_TX_QUEUE_SIZE),
-                        libvirt_utils.version_to_string(
-                            MIN_QEMU_TX_QUEUE_SIZE))
-            tx = None
-        return rx, tx
-
-    def _set_config_VIFGeneric(self, instance, vif, conf, host):
+    def _set_config_VIFGeneric(self, instance, vif, conf):
         dev = vif.vif_name
-        designer.set_vif_host_backend_ethernet_config(conf, dev, host)
+        designer.set_vif_host_backend_ethernet_config(conf, dev)
 
-    def _set_config_VIFBridge(self, instance, vif, conf, host=None):
+    def _set_config_VIFBridge(self, instance, vif, conf):
         conf.net_type = "bridge"
         conf.source_dev = vif.bridge_name
         conf.target_dev = vif.vif_name
 
-    def _set_config_VIFOpenVSwitch(self, instance, vif, conf, host=None):
+    def _set_config_VIFOpenVSwitch(self, instance, vif, conf):
         conf.net_type = "bridge"
         conf.source_dev = vif.bridge_name
         conf.target_dev = vif.vif_name
         self._set_config_VIFPortProfile(instance, vif, conf)
 
-    def _set_config_VIFVHostUser(self, instance, vif, conf, host=None):
+    def _set_config_VIFVHostUser(self, instance, vif, conf):
         # TODO(sahid): We should never configure a driver backend for
         # vhostuser interface. Specifically override driver to use
         # None. This can be removed when get_base_config will be fixed
         # and rewrite to set the correct backend.
         conf.driver_name = None
 
-        rx_queue_size, tx_queue_size = self._get_virtio_queue_sizes(host)
         designer.set_vif_host_backend_vhostuser_config(
-            conf, vif.mode, vif.path, rx_queue_size, tx_queue_size)
+            conf, vif.mode, vif.path, CONF.libvirt.rx_queue_size,
+            CONF.libvirt.tx_queue_size, vif.vif_name)
 
-    def _set_config_VIFHostDevice(self, instance, vif, conf, host=None):
+    def _set_config_VIFHostDevice(self, instance, vif, conf):
         if vif.dev_type == osv_fields.VIFHostDeviceDevType.ETHERNET:
             # This sets the required fields for an <interface type='hostdev'>
             # section in a libvirt domain (by using a subset of hw_veb's
@@ -546,7 +495,7 @@ class LibvirtGenericVIFDriver(object):
                 _('Unsupported VIF port profile type %s') % profile_name)
 
     def _get_config_os_vif(self, instance, vif, image_meta, inst_type,
-                           virt_type, host, vnic_type):
+                           virt_type, vnic_type):
         """Get the domain config for a VIF
 
         :param instance: nova.objects.Instance
@@ -554,7 +503,6 @@ class LibvirtGenericVIFDriver(object):
         :param image_meta: nova.objects.ImageMeta
         :param inst_type: nova.objects.Flavor
         :param virt_type: virtualization type
-        :param host: nova.virt.libvirt.host.Host
         :param vnic_type: vnic type
 
         :returns: nova.virt.libvirt.config.LibvirtConfigGuestInterface
@@ -562,20 +510,19 @@ class LibvirtGenericVIFDriver(object):
 
         # Do the config that's common to all vif types
         conf = self.get_base_config(instance, vif.address, image_meta,
-                                    inst_type, virt_type, vnic_type,
-                                    host)
+                                    inst_type, virt_type, vnic_type)
 
         # Do the VIF type specific config
         if isinstance(vif, osv_vifs.VIFGeneric):
-            self._set_config_VIFGeneric(instance, vif, conf, host)
+            self._set_config_VIFGeneric(instance, vif, conf)
         elif isinstance(vif, osv_vifs.VIFBridge):
-            self._set_config_VIFBridge(instance, vif, conf, host)
+            self._set_config_VIFBridge(instance, vif, conf)
         elif isinstance(vif, osv_vifs.VIFOpenVSwitch):
-            self._set_config_VIFOpenVSwitch(instance, vif, conf, host)
+            self._set_config_VIFOpenVSwitch(instance, vif, conf)
         elif isinstance(vif, osv_vifs.VIFVHostUser):
-            self._set_config_VIFVHostUser(instance, vif, conf, host)
+            self._set_config_VIFVHostUser(instance, vif, conf)
         elif isinstance(vif, osv_vifs.VIFHostDevice):
-            self._set_config_VIFHostDevice(instance, vif, conf, host)
+            self._set_config_VIFHostDevice(instance, vif, conf)
         else:
             raise exception.InternalError(
                 _("Unsupported VIF type %s") % vif.obj_name())
@@ -585,14 +532,12 @@ class LibvirtGenericVIFDriver(object):
         if vif.obj_name() not in ('VIFVHostUser', 'VIFHostDevice'):
             designer.set_vif_bandwidth_config(conf, inst_type)
 
-        if ('network' in vif and 'mtu' in vif.network and
-                self._has_min_version_for_mtu(host)):
+        if 'network' in vif and 'mtu' in vif.network:
             designer.set_vif_mtu_config(conf, vif.network.mtu)
 
         return conf
 
-    def get_config(self, instance, vif, image_meta,
-                   inst_type, virt_type, host):
+    def get_config(self, instance, vif, image_meta, inst_type, virt_type):
         vif_type = vif['type']
         vnic_type = vif['vnic_type']
 
@@ -612,11 +557,10 @@ class LibvirtGenericVIFDriver(object):
         vif_obj = os_vif_util.nova_to_osvif_vif(vif)
         if vif_obj is not None:
             return self._get_config_os_vif(instance, vif_obj, image_meta,
-                                           inst_type, virt_type, host,
-                                           vnic_type)
+                                           inst_type, virt_type, vnic_type)
 
         # Legacy non-os-vif codepath
-        args = (instance, vif, image_meta, inst_type, virt_type, host)
+        args = (instance, vif, image_meta, inst_type, virt_type)
         if vif_type == network_model.VIF_TYPE_IOVISOR:
             return self.get_config_iovisor(*args)
         elif vif_type == network_model.VIF_TYPE_802_QBG:
@@ -652,7 +596,7 @@ class LibvirtGenericVIFDriver(object):
                 vnic_mac, device_id, fabric,
                 network_model.VIF_TYPE_IB_HOSTDEV, pci_slot)
         except processutils.ProcessExecutionError:
-            LOG.exception(_("Failed while plugging ib hostdev vif"),
+            LOG.exception("Failed while plugging ib hostdev vif",
                           instance=instance)
 
     def plug_hw_veb(self, instance, vif):
@@ -695,7 +639,7 @@ class LibvirtGenericVIFDriver(object):
             nova.privsep.linux_net.create_tap_dev(dev)
             nova.privsep.libvirt.plug_midonet_vif(port_id, dev)
         except processutils.ProcessExecutionError:
-            LOG.exception(_("Failed while plugging vif"), instance=instance)
+            LOG.exception("Failed while plugging vif", instance=instance)
 
     def plug_iovisor(self, instance, vif):
         """Plug using PLUMgrid IO Visor Driver
@@ -712,7 +656,7 @@ class LibvirtGenericVIFDriver(object):
             nova.privsep.libvirt.plug_plumgrid_vif(
                 dev, iface_id, vif['address'], net_id, tenant_id)
         except processutils.ProcessExecutionError:
-            LOG.exception(_("Failed while plugging vif"), instance=instance)
+            LOG.exception("Failed while plugging vif", instance=instance)
 
     def plug_tap(self, instance, vif):
         """Plug a VIF_TYPE_TAP virtual interface."""
@@ -722,8 +666,7 @@ class LibvirtGenericVIFDriver(object):
         vif_model = self.get_vif_model(image_meta=image_meta)
         # TODO(ganso): explore whether multiqueue works for other vif models
         # that go through this code path.
-        multiqueue = (instance.get_flavor().vcpus > 1 and
-                      self._requests_multiqueue(image_meta) and
+        multiqueue = (self._requests_multiqueue(image_meta) and
                       vif_model == network_model.VIF_MODEL_VIRTIO)
         nova.privsep.linux_net.create_tap_dev(dev, mac, multiqueue=multiqueue)
         network = vif.get('network')
@@ -794,7 +737,7 @@ class LibvirtGenericVIFDriver(object):
         try:
             nova.privsep.libvirt.unplug_infiniband_vif(fabric, vnic_mac)
         except Exception:
-            LOG.exception(_("Failed while unplugging ib hostdev vif"))
+            LOG.exception("Failed while unplugging ib hostdev vif")
 
     def unplug_hw_veb(self, instance, vif):
         # TODO(sean-k-mooney): remove in Train after backporting 0 mac
@@ -825,7 +768,7 @@ class LibvirtGenericVIFDriver(object):
             nova.privsep.libvirt.unplug_midonet_vif(port_id)
             nova.privsep.linux_net.delete_net_dev(dev)
         except processutils.ProcessExecutionError:
-            LOG.exception(_("Failed while unplugging vif"), instance=instance)
+            LOG.exception("Failed while unplugging vif", instance=instance)
 
     def unplug_tap(self, instance, vif):
         """Unplug a VIF_TYPE_TAP virtual interface."""
@@ -833,7 +776,7 @@ class LibvirtGenericVIFDriver(object):
         try:
             nova.privsep.linux_net.delete_net_dev(dev)
         except processutils.ProcessExecutionError:
-            LOG.exception(_("Failed while unplugging vif"), instance=instance)
+            LOG.exception("Failed while unplugging vif", instance=instance)
 
     def unplug_iovisor(self, instance, vif):
         """Unplug using PLUMgrid IO Visor Driver
@@ -846,7 +789,7 @@ class LibvirtGenericVIFDriver(object):
             nova.privsep.libvirt.unplug_plumgrid_vif(dev)
             nova.privsep.linux_net.delete_net_dev(dev)
         except processutils.ProcessExecutionError:
-            LOG.exception(_("Failed while unplugging vif"), instance=instance)
+            LOG.exception("Failed while unplugging vif", instance=instance)
 
     def _unplug_os_vif(self, instance, vif):
         instance_info = os_vif_util.nova_to_osvif_instance(instance)
